local L = AceLibrary("AceLocale-2.2"):new("GridStatusControlPipes")

local AceOO = AceLibrary("AceOO-2.0")

local PipeGuard = AceOO.Class()


------------------------------------
--      General pipe guard        --
------------------------------------



function PipeGuard.prototype:init(pipe, id, db, options)
    PipeGuard.super.prototype.init(self, pipe, id, db, options)
    print("new guard")
    self.options = options
    self.db = db
    self.pipe = pipe
    self.isOpen = true
end

function PipeGuard.prototype:Destroy()
end

function PipeGuard.prototype:IsOpen(guid, color, text, value, maxValue, texture, start, duration, stack, ...)
    return self.isOpen
end

function PipeGuard:GetName()
    return "not used"
end

function PipeGuard:RegisterGuardType(name, class)
    self.guardTypes = self.guardTypes or {}
    self.guardTypes[name] = class
    
    GridPipeClass:RegisterGuard(name, class)
end


------------------------------------
--             KeyGuard           --
------------------------------------

local KeyGuard = AceOO.Class(PipeGuard)
PipeGuard:RegisterGuardType("Key guard", KeyGuard)

BINDING_HEADER_PIPE_GUARDS = L["Grid status control pipes - guard keys"]
BINDING_NAME_guard_key_1 = L["Guard key "].."1"
BINDING_NAME_guard_key_2 = L["Guard key "].."2"

function KeyGuard.prototype:init(pipe, id, db, options)
    KeyGuard.super.prototype.init(self, pipe, id, db, options)
    print("new key guard")
    self.db.key = self.db.key or 1
    KeyGuard:RegisterWithKey(self, self.db.key)
    
    self.options.args = {
        ["key"] = {
            type = "text",
            name = L["Key"],
            desc = L["The key binding to use."],
            get = function ()
                return self.db.key
                end,
            set = function(v)
                KeyGuard:UnregisterWithKey(self, self.db.key)
                self.db.key = v
                print("key is "..self.db.key)
                KeyGuard:RegisterWithKey(self, v)
            end,
            validate = {[1] = 1, [2] = 2},
            validateDesc = {[1] = L["Guard key "].."1", [2] = L["Guard key "].."2"},
            order = 10,
        },
        ["key_binding"] = {
            type = "text",
            name = L["Current binding"],
            desc = L["Sets the current binding for this key."],
            get = function() return GetBindingKey("guard_key_"..self.db.key) end,
            usage = L["<key>"],
            set = function(v) 
                    local oldKey = GetBindingKey("guard_key_"..self.db.key)    
                    local suc = SetBinding(v, "guard_key_"..self.db.key)
                    if suc then
                        if oldKey then SetBinding(oldKey) end
                        SaveBindings(2)
                    end     
                  end,
            validate = "keybinding",
            order = 12,            
        },
    }
end

function KeyGuard.prototype:OnKeyUp()
    local gone = GetTime() - (self.keyDownTime or 0)
    if gone > 0.3 then
        self.isOpen = not self.isOpen
        self.pipe:UpdateAll()
    end
end

function KeyGuard.prototype:OnKeyDown()
    self.isOpen = not self.isOpen
    self.pipe:UpdateAll()
    
    self.keyDownTime = GetTime()
end

function KeyGuard.prototype:Destroy()
    KeyGuard:UnregisterWithKey(self, self.db.key)
end

function KeyGuard:RegisterWithKey(guard, key)
    local keyGuards = self:RegisteredGuards(key)
    
    tinsert(keyGuards, guard)
end

function KeyGuard:RegisteredGuards(key)
    local reg = self.registry
    if not reg then
        reg = {}
        self.registry = reg
    end
    
    local keyGuards = reg[key]
    if not keyGuards then
        keyGuards = {}
        reg[key] = keyGuards
    end

    return keyGuards
end

function KeyGuard:UnregisterWithKey(guard, key)
    local keyGuards = self:RegisteredGuards(key)
    
    for i, handle in ipairs(keyGuards) do
        if guard == handle then
            tremove(keyGuards, i)
            break
        end
    end
end

function KeyGuard:GuardKeyDown(key)
    local keyGuards = self:RegisteredGuards(key)
    for i, handle in ipairs(keyGuards) do
        handle:OnKeyDown()
    end
end

function KeyGuard:GuardKeyUp(key)
    local keyGuards = self:RegisteredGuards(key)
    for i, handle in ipairs(keyGuards) do
        handle:OnKeyUp()
    end
end

function KeyGuard:GetName()
    return "Key guard"
end






------------------------------------
--          CombatGuard           --
------------------------------------

local AceEvent = AceLibrary("AceEvent-2.0")

local CombatGuard = AceOO.Class(PipeGuard, AceEvent)
PipeGuard:RegisterGuardType("Combat guard", CombatGuard)

function CombatGuard.prototype:init(pipe, id, db, options)
    CombatGuard.super.prototype.init(self, pipe, id, db, options)
    
    self.db.inCombat = self.db.inCombat or false
    self.combat = false
    self.options.args = {
        ["mode"] = {
            type = "toggle",
            name = L["In combat piping"],
            desc = L["If enabled, the guard is open when in combat. Otherwise the guard is open out of combat."],
            get = function() return self.db.inCombat end,
            set = function(v) self.db.inCombat = v; self.pipe:UpdateAll() end,
        },
    }
    
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function CombatGuard.prototype:PLAYER_REGEN_DISABLED()
    self.combat = true
    self.pipe:UpdateAll()
end

function CombatGuard.prototype:PLAYER_REGEN_ENABLED()
    self.combat = false
    self.pipe:UpdateAll()
end

function CombatGuard.prototype:IsOpen(guid, color, text, value, maxValue, texture, start, duration, stack, ...)
    return self.combat == self.db.inCombat
end

function CombatGuard.prototype:Destroy()
    self:UnregisterEvent("PLAYER_REGEN_DISABLED")
    self:UnregisterEvent("PLAYER_REGEN_ENABLED")
end

function PipeGuard:GetName()
    return "Combat guard"
end




------------------------------------
--         ThresholdGuard         --
------------------------------------

local ThresholdGuard = AceOO.Class(PipeGuard)
PipeGuard:RegisterGuardType("Threshold guard", ThresholdGuard)

function ThresholdGuard.prototype:init(pipe, id, db, options)
    ThresholdGuard.super.prototype.init(self, pipe, id, db, options)
    
    self.db.thresh = self.db.thresh or 80
    self.options.args = {
        ["threshold"] = {
            type = "range",
            name = L["Threshold (in %)"],
            desc = L["Sets the threshold in percent when the guard should be open."],
            max = 100,
            min = 0,
            step = 1,
            get = function ()
                      return self.db.thresh
                  end,
            set = function (v)
                      self.db.thresh = v
                      self.pipe:UpdateAll()
                  end,
        }
    }
    
end

function ThresholdGuard.prototype:IsOpen(guid, color, text, value, maxValue, texture, start, duration, stack, ...)
    return not value or not maxValue or (value / maxValue * 100 < self.db.thresh)
end


function ThresholdGuard:GetName()
    return "Threshold guard"
end



PipeGuardClass = PipeGuard
KeyGuardClass = KeyGuard
CombatGuardClass = CombatGuard